package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestConfiguration;
import at.ac.uibk.gitsearch.service.ZipRepackagingServiceTest.MockedSingletonTreeNodeForFiltering;
import java.io.IOException;
import java.io.InputStream;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import org.elasticsearch.node.NodeValidationException;
import org.junit.Assert;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;

@SpringBootTest(classes = GitsearchApp.class)
@WithMockUser(value = "test", authorities = "sharing")
public class ZipRepackagingServiceIT {

    private static final String TEST_ZIP_LOCATION = "./testData/junit-quality-tests-exercise-master.zip";

    @Autowired
    private ZipRepackagingService zipRepackagingService;

    private static final Logger log = LoggerFactory.getLogger(ZipRepackagingServiceIT.class);

    @Autowired
    private ElasticSearchTestConfiguration elasticSearchTestConfiguration;

    static ElasticSearchTestConfiguration staticElasticSearchTestConfiguration;

    @BeforeEach // must be started as BeforeEach, in order to autowire the ElasticSearchTestConfiguration
    @Timeout(value = 2, unit = TimeUnit.MINUTES)
    public void setUpESServer() throws IOException, NodeValidationException {
        if (staticElasticSearchTestConfiguration == null) {
            elasticSearchTestConfiguration.startTestNode();
            staticElasticSearchTestConfiguration = elasticSearchTestConfiguration;
        }
    }

    @AfterAll
    public static void shutDownESServer() {
        if (staticElasticSearchTestConfiguration != null) {
            staticElasticSearchTestConfiguration.stopTestContainer();
            staticElasticSearchTestConfiguration = null;
        }
    }

    @Test
    public void testSimpleRepackage() throws IOException {
        final InputStream resourceStream = this.getClass().getResourceAsStream(TEST_ZIP_LOCATION);

        Assert.assertNotNull("cannot find resource at " + TEST_ZIP_LOCATION, resourceStream);
        final ZipInputStream testZip = new ZipInputStream(resourceStream);

        final InputStream testRepackagedZipIS = zipRepackagingService.rePackageGitLabProjectZip(
            testZip,
            "from " + TEST_ZIP_LOCATION,
            null,/* no filtering */
            ""
        );

        final ZipInputStream testRepackagedZip = new ZipInputStream(testRepackagedZipIS);

        ZipEntry entry = testRepackagedZip.getNextEntry();

        while (entry != null) {
            log.info("found entry {} (directory: {})", entry.getName(), entry.isDirectory());
            Assert.assertTrue("original folder not chopped off?", !entry.getName().startsWith("junit-quality-tests-exercise-master"));
            entry = testRepackagedZip.getNextEntry();
        }

        testRepackagedZip.close();
    }

    @Test
    public void testFilteredRepackage() throws IOException {
        final MockedSingletonTreeNodeForFiltering filterOut = new MockedSingletonTreeNodeForFiltering("solution", "tests");

        // first check whether this is good test data
        final ZipInputStream originalZip = new ZipInputStream(this.getClass().getResourceAsStream(TEST_ZIP_LOCATION));

        boolean found = false;
        for (ZipEntry entry = originalZip.getNextEntry(); entry != null; entry = originalZip.getNextEntry()) {
            found = found || isPathPrefix(filterOut.getFilterStrings(), "junit-quality-tests-exercise-master/", entry);
            if (found) break;
        }
        Assert.assertTrue("test zip should contain entries to filter away?", found);

        final InputStream resourceStream = this.getClass().getResourceAsStream(TEST_ZIP_LOCATION);

        Assert.assertNotNull("cannot find resource at " + TEST_ZIP_LOCATION, resourceStream);
        final ZipInputStream testZip = new ZipInputStream(resourceStream);

        final InputStream testRepackagedZipIS = zipRepackagingService.rePackageGitLabProjectZip(
            testZip,
            "from " + TEST_ZIP_LOCATION,
            Optional.of(filterOut),
            ""
        );

        final ZipInputStream testRepackagedZip = new ZipInputStream(testRepackagedZipIS);

        for (ZipEntry entry = testRepackagedZip.getNextEntry(); entry != null; entry = testRepackagedZip.getNextEntry()) {
            boolean foundPrefix = isPathPrefix(filterOut.getFilterStrings(), "", entry);
            Assert.assertFalse("original folder starts with excepted directory?", foundPrefix);
        }

        testRepackagedZip.close();
    }

    /**
     * tests for the path prefix in filterOut
     * @param filterOut the list of path prefixes to filter out
     * @param basePrefix a basePrefix to ignore
     * @param entry the zip entry
     * @return true if one of the path prefixes matches
     */
    private boolean isPathPrefix(final String[] filterOut, String basePrefix, ZipEntry entry) {
        boolean found = false;
        String fileName = entry.getName();
        if (fileName.startsWith(basePrefix)) fileName = fileName.substring(basePrefix.length());
        for (String filter : filterOut) {
            found = found || fileName.startsWith(filter); // this is slighlty lazy, because we should check for real path
            if (found) break;
        }
        return found;
    }

    /**
     * tests for subpackages
     * @throws IOException
     */
    @Test
    public void testSubRepackage() throws IOException {
        final InputStream resourceStream = this.getClass().getResourceAsStream(TEST_ZIP_LOCATION);

        Assert.assertNotNull("cannot find resource at " + TEST_ZIP_LOCATION, resourceStream);
        final ZipInputStream testZip = new ZipInputStream(resourceStream);

        final InputStream testRepackagedZipIS = zipRepackagingService.rePackageGitLabProjectZip(
            testZip,
            "from " + TEST_ZIP_LOCATION,
            null,/* no filtering */
            "exercise"
        );

        final ZipInputStream testRepackagedZip = new ZipInputStream(testRepackagedZipIS);

        for (ZipEntry entry = testRepackagedZip.getNextEntry(); entry != null; entry = testRepackagedZip.getNextEntry()) {
            boolean foundPrefix = isPathPrefix(new String[] { "solution", "tests", "test" }, "", entry);
            Assert.assertFalse("original folder starts with excepted directory?", foundPrefix);
        }

        testRepackagedZip.close();
    }
}
